package kr.co.vividnext.sodalive.admin.content

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.AudioContent
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.hashtag.QAudioContentHashTag.audioContentHashTag
import kr.co.vividnext.sodalive.content.hashtag.QHashTag.hashTag
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AdminContentRepository : JpaRepository<AudioContent, Long>, AdminAudioContentQueryRepository

interface AdminAudioContentQueryRepository {
    fun getAudioContentTotalCount(searchWord: String = ""): Int
    fun getAudioContentList(offset: Long, limit: Long, searchWord: String = ""): List<GetAdminContentListItem>
    fun getHashTagList(audioContentId: Long): List<String>
}

class AdminAudioContentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AdminAudioContentQueryRepository {
    override fun getAudioContentTotalCount(searchWord: String): Int {
        var where = audioContent.duration.isNotNull
            .and(audioContent.member.isNotNull)
            .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))

        if (searchWord.trim().length > 1) {
            where = where.and(
                audioContent.title.contains(searchWord)
                    .or(audioContent.member.nickname.contains(searchWord))
            )
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .where(where)
            .fetch()
            .size
    }

    override fun getAudioContentList(offset: Long, limit: Long, searchWord: String): List<GetAdminContentListItem> {
        var where = audioContent.duration.isNotNull
            .and(audioContent.member.isNotNull)
            .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))

        if (searchWord.trim().length > 1) {
            where = where.and(
                audioContent.title.contains(searchWord)
                    .or(audioContent.member.nickname.contains(searchWord))
            )
        }

        return queryFactory
            .select(
                QGetAdminContentListItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.detail,
                    audioContentCuration.title,
                    audioContentCuration.id.nullif(0),
                    audioContent.coverImage,
                    audioContent.member!!.nickname,
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.isAdult,
                    audioContent.duration,
                    audioContent.content,
                    formattedDateExpression(audioContent.createdAt),
                    formattedDateExpression(audioContent.releaseDate, "%Y-%m-%d %H:%i")
                )
            )
            .from(audioContent)
            .leftJoin(audioContent.curation, audioContentCuration)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContent.id.desc())
            .fetch()
    }

    override fun getHashTagList(audioContentId: Long): List<String> {
        return queryFactory
            .select(hashTag.tag)
            .from(audioContentHashTag)
            .innerJoin(audioContentHashTag.hashTag, hashTag)
            .innerJoin(audioContentHashTag.audioContent, audioContent)
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContentHashTag.audioContent.id.eq(audioContentId))
            )
            .fetch()
    }

    private fun formattedDateExpression(
        dateTime: DateTimePath<LocalDateTime>,
        format: String = "%Y-%m-%d"
    ): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTime,
                "UTC",
                "Asia/Seoul"
            ),
            format
        )
    }
}
