package kr.co.vividnext.sodalive.admin.member.tag

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.tag.CreatorTag
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminMemberTagService(
    private val repository: AdminMemberTagRepository,
    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    @Transactional
    fun uploadTagImage(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateMemberTagRequest::class.java)
        tagExistCheck(request)

        val fileName = generateFileName()
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "creator_tag/$fileName"
        )
        return createTag(tag = request.tag, imagePath = imagePath, isAdult = request.isAdult)
    }

    private fun tagExistCheck(request: CreateMemberTagRequest) {
        repository.findByTag(request.tag)?.let { throw SodaException("이미 등록된 태그 입니다.") }
    }

    private fun createTag(tag: String, imagePath: String, isAdult: Boolean) {
        repository.save(
            CreatorTag(
                tag = tag,
                isAdult = isAdult,
                image = imagePath
            )
        )
    }

    @Transactional
    fun deleteTag(id: Long) {
        val creatorTag = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        creatorTag.tag = "${creatorTag.tag}_deleted"
        creatorTag.isActive = false
    }

    @Transactional
    fun modifyTag(id: Long, image: MultipartFile?, requestString: String) {
        val creatorTag = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        val request = objectMapper.readValue(requestString, CreateMemberTagRequest::class.java)
        creatorTag.tag = request.tag
        creatorTag.isAdult = request.isAdult

        if (image != null) {
            val fileName = generateFileName()
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "creator_tag/$fileName"
            )
            creatorTag.image = imagePath
        }
    }

    @Transactional
    fun updateTagOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val tag = repository.findByIdOrNull(ids[index])

            if (tag != null) {
                tag.orders = index + 1
            }
        }
    }
}
