package kr.co.vividnext.sodalive.can.charge

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.bootpay.Bootpay
import kr.co.vividnext.sodalive.can.CanRepository
import kr.co.vividnext.sodalive.can.charge.event.ChargeSpringEvent
import kr.co.vividnext.sodalive.can.coupon.CanCouponNumberRepository
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.RequestBody.Companion.toRequestBody
import org.json.JSONObject
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.http.HttpHeaders
import org.springframework.security.core.userdetails.User
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
@Transactional(readOnly = true)
class ChargeService(
    private val chargeRepository: ChargeRepository,
    private val canRepository: CanRepository,
    private val memberRepository: MemberRepository,
    private val couponNumberRepository: CanCouponNumberRepository,

    private val objectMapper: ObjectMapper,
    private val okHttpClient: OkHttpClient,
    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${bootpay.application-id}")
    private val bootpayApplicationId: String,
    @Value("\${bootpay.private-key}")
    private val bootpayPrivateKey: String,
    @Value("\${apple.iap-verify-sandbox-url}")
    private val appleInAppVerifySandBoxUrl: String,
    @Value("\${apple.iap-verify-url}")
    private val appleInAppVerifyUrl: String
) {

    @Transactional
    fun chargeByCoupon(couponNumber: String, member: Member) {
        val canCouponNumber = couponNumberRepository.findByCouponNumber(couponNumber = couponNumber)
            ?: throw SodaException("잘못된 쿠폰번호입니다.\n고객센터로 문의해 주시기 바랍니다.")

        if (canCouponNumber.member != null) {
            throw SodaException("이미 사용한 쿠폰번호 입니다.")
        }

        canCouponNumber.member = member

        val coupon = canCouponNumber.canCoupon!!
        val couponCharge = Charge(0, coupon.can, status = ChargeStatus.COUPON)
        couponCharge.title = "${coupon.can} 캔"
        couponCharge.member = member

        val payment = Payment(
            status = PaymentStatus.COMPLETE,
            paymentGateway = PaymentGateway.PG
        )
        payment.method = coupon.couponName
        couponCharge.payment = payment
        chargeRepository.save(couponCharge)

        member.charge(0, coupon.can, "pg")
    }

    @Transactional
    fun charge(member: Member, request: ChargeRequest): ChargeResponse {
        val can = canRepository.findByIdOrNull(request.canId)
            ?: throw SodaException("잘못된 요청입니다\n앱 종료 후 다시 시도해 주세요.")

        val charge = Charge(can.can, can.rewardCan)
        charge.title = can.title
        charge.member = member
        charge.can = can

        val payment = Payment(paymentGateway = request.paymentGateway)
        payment.price = can.price.toDouble()
        charge.payment = payment

        chargeRepository.save(charge)

        return ChargeResponse(chargeId = charge.id!!)
    }

    @Transactional
    fun verify(user: User, verifyRequest: VerifyRequest) {
        val charge = chargeRepository.findByIdOrNull(verifyRequest.orderId.toLong())
            ?: throw SodaException("결제정보에 오류가 있습니다.")
        val member = memberRepository.findByEmail(user.username)
            ?: throw SodaException("로그인 정보를 확인해주세요.")

        if (charge.payment!!.paymentGateway == PaymentGateway.PG) {
            val bootpay = Bootpay(bootpayApplicationId, bootpayPrivateKey)

            try {
                bootpay.accessToken
                val verifyResult = objectMapper.convertValue(
                    bootpay.getReceipt(verifyRequest.receiptId),
                    VerifyResult::class.java
                )

                if (verifyResult.status == 1 && verifyResult.price == charge.can?.price) {
                    charge.payment?.receiptId = verifyResult.receiptId
                    charge.payment?.method = verifyResult.method
                    charge.payment?.status = PaymentStatus.COMPLETE
                    member.charge(charge.chargeCan, charge.rewardCan, "pg")

                    applicationEventPublisher.publishEvent(
                        ChargeSpringEvent(
                            chargeId = charge.id!!,
                            memberId = member.id!!
                        )
                    )
                } else {
                    throw SodaException("결제정보에 오류가 있습니다.")
                }
            } catch (e: Exception) {
                throw SodaException("결제정보에 오류가 있습니다.")
            }
        } else {
            throw SodaException("결제정보에 오류가 있습니다.")
        }
    }

    @Transactional
    fun appleCharge(member: Member, request: AppleChargeRequest): ChargeResponse {
        val charge = Charge(request.chargeCan, 0)
        charge.title = request.title
        charge.member = member

        val payment = Payment(paymentGateway = request.paymentGateway)
        payment.price = if (request.price != null) {
            request.price!!
        } else {
            0.toDouble()
        }

        payment.locale = request.locale

        charge.payment = payment

        chargeRepository.save(charge)

        return ChargeResponse(chargeId = charge.id!!)
    }

    @Transactional
    fun appleVerify(user: User, verifyRequest: AppleVerifyRequest) {
        val charge = chargeRepository.findByIdOrNull(verifyRequest.chargeId)
            ?: throw SodaException("결제정보에 오류가 있습니다.")
        val member = memberRepository.findByEmail(user.username)
            ?: throw SodaException("로그인 정보를 확인해주세요.")

        if (charge.payment!!.paymentGateway == PaymentGateway.APPLE_IAP) {
            // 검증로직
            if (requestRealServerVerify(verifyRequest)) {
                charge.payment?.receiptId = verifyRequest.receiptString
                charge.payment?.method = "애플(인 앱 결제)"
                charge.payment?.status = PaymentStatus.COMPLETE
                member.charge(charge.chargeCan, charge.rewardCan, "ios")

                applicationEventPublisher.publishEvent(
                    ChargeSpringEvent(
                        chargeId = charge.id!!,
                        memberId = member.id!!
                    )
                )
            } else {
                throw SodaException("결제정보에 오류가 있습니다.")
            }
        } else {
            throw SodaException("결제정보에 오류가 있습니다.")
        }
    }

    private fun requestRealServerVerify(verifyRequest: AppleVerifyRequest): Boolean {
        val body = JSONObject()
        body.put("receipt-data", verifyRequest.receiptString)
        val request = Request.Builder()
            .url(appleInAppVerifyUrl)
            .addHeader(HttpHeaders.CONTENT_TYPE, "application/json")
            .post(body.toString().toRequestBody("application/json".toMediaTypeOrNull()))
            .build()

        val response = okHttpClient.newCall(request).execute()
        if (response.isSuccessful) {
            val responseString = response.body?.string()
            if (responseString != null) {
                val verifyResult = objectMapper.readValue(responseString, AppleVerifyResponse::class.java)
                return when (verifyResult.status) {
                    0 -> {
                        true
                    }

                    21007 -> {
                        requestSandboxServerVerify(verifyRequest)
                    }

                    else -> {
                        throw SodaException("결제정보에 오류가 있습니다.")
                    }
                }
            } else {
                throw SodaException("결제를 완료하지 못했습니다.")
            }
        } else {
            throw SodaException("결제를 완료하지 못했습니다.")
        }
    }

    private fun requestSandboxServerVerify(verifyRequest: AppleVerifyRequest): Boolean {
        val body = JSONObject()
        body.put("receipt-data", verifyRequest.receiptString)
        val request = Request.Builder()
            .url(appleInAppVerifySandBoxUrl)
            .addHeader(HttpHeaders.CONTENT_TYPE, "application/json")
            .post(body.toString().toRequestBody("application/json".toMediaTypeOrNull()))
            .build()

        val response = okHttpClient.newCall(request).execute()
        if (response.isSuccessful) {
            val responseString = response.body?.string()
            if (responseString != null) {
                val verifyResult = objectMapper.readValue(responseString, AppleVerifyResponse::class.java)
                return when (verifyResult.status) {
                    0 -> {
                        true
                    }

                    else -> {
                        throw SodaException("결제정보에 오류가 있습니다.")
                    }
                }
            } else {
                throw SodaException("결제를 완료하지 못했습니다.")
            }
        } else {
            throw SodaException("결제를 완료하지 못했습니다.")
        }
    }
}
