package kr.co.vividnext.sodalive.can.payment

import kr.co.vividnext.sodalive.can.CanRepository
import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.SpentCan
import kr.co.vividnext.sodalive.can.use.TotalSpentCan
import kr.co.vividnext.sodalive.can.use.UseCan
import kr.co.vividnext.sodalive.can.use.UseCanCalculate
import kr.co.vividnext.sodalive.can.use.UseCanCalculateRepository
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.can.use.UseCanRepository
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContent
import kr.co.vividnext.sodalive.content.order.Order
import kr.co.vividnext.sodalive.live.room.LiveRoom
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class CanPaymentService(
    private val repository: CanRepository,
    private val memberRepository: MemberRepository,
    private val chargeRepository: ChargeRepository,
    private val useCanRepository: UseCanRepository,
    private val useCanCalculateRepository: UseCanCalculateRepository
) {
    @Transactional
    fun spendCan(
        memberId: Long,
        needCan: Int,
        canUsage: CanUsage,
        liveRoom: LiveRoom? = null,
        order: Order? = null,
        audioContent: AudioContent? = null,
        container: String
    ) {
        val member = memberRepository.findByIdOrNull(id = memberId)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")
        val useRewardCan = spendRewardCan(member, needCan, container)
        val useChargeCan = if (needCan - useRewardCan.total > 0) {
            spendChargeCan(member, needCan = needCan - useRewardCan.total, container = container)
        } else {
            null
        }

        if (needCan - useRewardCan.total - (useChargeCan?.total ?: 0) > 0) {
            throw SodaException(
                "${needCan - useRewardCan.total - (useChargeCan?.total ?: 0)} " +
                    "캔이 부족합니다. 충전 후 이용해 주세요."
            )
        }

        if (!useRewardCan.verify() || useChargeCan?.verify() == false) {
            throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")
        }

        val useCan = UseCan(
            canUsage = canUsage,
            can = useChargeCan?.total ?: 0,
            rewardCan = useRewardCan.total
        )

        var recipientId: Long? = null
        if (canUsage == CanUsage.LIVE && liveRoom != null) {
            recipientId = liveRoom.member!!.id!!
            useCan.room = liveRoom
            useCan.member = member
        } else if (canUsage == CanUsage.CHANGE_NICKNAME) {
            useCan.member = member
        } else if (canUsage == CanUsage.DONATION && liveRoom != null) {
            recipientId = liveRoom.member!!.id!!
            useCan.room = liveRoom
            useCan.member = member
        } else if (canUsage == CanUsage.ORDER_CONTENT && order != null) {
            recipientId = order.creator!!.id!!
            useCan.order = order
            useCan.member = member
            useCan.audioContent = order.audioContent
        } else if (canUsage == CanUsage.DONATION && audioContent != null) {
            recipientId = audioContent.member!!.id!!
            useCan.audioContent = audioContent
            useCan.member = member
        } else if (canUsage == CanUsage.SPIN_ROULETTE && liveRoom != null) {
            recipientId = liveRoom.member!!.id!!
            useCan.room = liveRoom
            useCan.member = member
        } else {
            throw SodaException("잘못된 요청입니다.")
        }

        useCanRepository.save(useCan)

        setUseCanCalculate(recipientId, useRewardCan, useChargeCan, useCan, paymentGateway = PaymentGateway.PG)
        setUseCanCalculate(
            recipientId,
            useRewardCan,
            useChargeCan,
            useCan,
            paymentGateway = PaymentGateway.POINT_CLICK_AD
        )
        setUseCanCalculate(
            recipientId,
            useRewardCan,
            useChargeCan,
            useCan,
            paymentGateway = PaymentGateway.GOOGLE_IAP
        )
        setUseCanCalculate(
            recipientId,
            useRewardCan,
            useChargeCan,
            useCan,
            paymentGateway = PaymentGateway.APPLE_IAP
        )
    }

    private fun setUseCanCalculate(
        recipientId: Long?,
        useRewardCan: TotalSpentCan,
        useChargeCan: TotalSpentCan?,
        useCan: UseCan,
        paymentGateway: PaymentGateway
    ) {
        val totalSpentRewardCan = useRewardCan.spentCans
            .filter { it.paymentGateway == paymentGateway }
            .fold(0) { sum, spentCans -> sum + spentCans.can }

        val useCanCalculate = if (useChargeCan != null) {
            val totalSpentChargeCan = useChargeCan.spentCans
                .filter { it.paymentGateway == paymentGateway }
                .fold(0) { sum, spentCans -> sum + spentCans.can }

            UseCanCalculate(
                can = totalSpentChargeCan + totalSpentRewardCan,
                paymentGateway = paymentGateway,
                status = UseCanCalculateStatus.RECEIVED
            )
        } else {
            UseCanCalculate(
                can = totalSpentRewardCan,
                paymentGateway = paymentGateway,
                status = UseCanCalculateStatus.RECEIVED
            )
        }

        if (useCanCalculate.can > 0) {
            useCanCalculate.useCan = useCan
            useCanCalculate.recipientCreatorId = recipientId
            useCanCalculateRepository.save(useCanCalculate)
        }
    }

    private fun spendRewardCan(member: Member, needCan: Int, container: String): TotalSpentCan {
        return if (needCan > 0) {
            val spentCans = mutableListOf<SpentCan>()
            var chargeId = 0L
            var total = 0

            while (needCan - total > 0) {
                val remainingNeedCan = needCan - total
                val charge = chargeRepository.getOldestChargeWhereRewardCanGreaterThan0(
                    chargeId,
                    member.id!!,
                    container
                ) ?: break

                if (charge.rewardCan >= remainingNeedCan) {
                    charge.rewardCan -= remainingNeedCan

                    when (charge.payment!!.paymentGateway) {
                        PaymentGateway.APPLE_IAP -> member.appleRewardCan -= remainingNeedCan
                        PaymentGateway.GOOGLE_IAP -> member.googleRewardCan -= remainingNeedCan
                        else -> member.pgRewardCan -= remainingNeedCan
                    }

                    total += remainingNeedCan

                    spentCans.add(
                        SpentCan(
                            paymentGateway = charge.payment!!.paymentGateway,
                            can = remainingNeedCan
                        )
                    )
                } else {
                    total += charge.rewardCan
                    spentCans.add(
                        SpentCan(
                            paymentGateway = charge.payment!!.paymentGateway,
                            can = charge.rewardCan
                        )
                    )

                    when (charge.payment!!.paymentGateway) {
                        PaymentGateway.APPLE_IAP -> member.appleRewardCan -= charge.rewardCan
                        PaymentGateway.GOOGLE_IAP -> member.googleRewardCan -= charge.rewardCan
                        else -> member.pgRewardCan -= charge.rewardCan
                    }

                    charge.rewardCan = 0
                }

                chargeId = charge.id!!
            }

            TotalSpentCan(spentCans, total)
        } else {
            TotalSpentCan(total = 0)
        }
    }

    private fun spendChargeCan(member: Member, needCan: Int, container: String): TotalSpentCan {
        return if (needCan > 0) {
            val spentCans = mutableListOf<SpentCan>()
            var chargeId = 0L
            var total = 0

            while (needCan - total > 0) {
                val remainingNeedCan = needCan - total
                val charge = chargeRepository.getOldestChargeWhereChargeCanGreaterThan0(
                    chargeId,
                    member.id!!,
                    container
                ) ?: break

                if (charge.chargeCan >= remainingNeedCan) {
                    charge.chargeCan -= remainingNeedCan

                    when (charge.payment!!.paymentGateway) {
                        PaymentGateway.APPLE_IAP -> member.appleChargeCan -= remainingNeedCan
                        PaymentGateway.GOOGLE_IAP -> member.googleChargeCan -= remainingNeedCan
                        else -> member.pgChargeCan -= remainingNeedCan
                    }

                    total += remainingNeedCan

                    spentCans.add(
                        SpentCan(
                            paymentGateway = charge.payment!!.paymentGateway,
                            can = remainingNeedCan
                        )
                    )
                } else {
                    total += charge.chargeCan
                    spentCans.add(
                        SpentCan(
                            paymentGateway = charge.payment!!.paymentGateway,
                            can = charge.chargeCan
                        )
                    )

                    when (charge.payment!!.paymentGateway) {
                        PaymentGateway.APPLE_IAP -> member.appleChargeCan -= charge.chargeCan
                        PaymentGateway.GOOGLE_IAP -> member.pgChargeCan -= charge.chargeCan
                        else -> member.pgChargeCan -= charge.chargeCan
                    }

                    charge.chargeCan = 0
                }

                chargeId = charge.id!!
            }

            TotalSpentCan(spentCans, total)
        } else {
            TotalSpentCan(total = 0)
        }
    }

    @Transactional
    fun refund(memberId: Long, roomId: Long) {
        val member = memberRepository.findByIdOrNull(memberId)
            ?: throw SodaException("잘못된 예약정보 입니다.")

        val useCan = repository.getCanUsedForLiveRoomNotRefund(
            memberId = memberId,
            roomId = roomId,
            canUsage = CanUsage.LIVE
        ) ?: throw SodaException("잘못된 예약정보 입니다.")
        useCan.isRefund = true

        val useCanCalculates = useCanCalculateRepository.findByUseCanIdAndStatus(useCan.id!!)
        useCanCalculates.forEach {
            it.status = UseCanCalculateStatus.REFUND
            val charge = Charge(0, it.can, status = ChargeStatus.REFUND_CHARGE)
            charge.title = "${it.can} 캔"
            charge.useCan = useCan

            when (it.paymentGateway) {
                PaymentGateway.GOOGLE_IAP -> member.googleRewardCan += charge.rewardCan
                PaymentGateway.APPLE_IAP -> member.appleRewardCan += charge.rewardCan
                else -> member.pgRewardCan += charge.rewardCan
            }
            charge.member = member

            val payment = Payment(
                status = PaymentStatus.COMPLETE,
                paymentGateway = it.paymentGateway
            )
            payment.method = "환불"
            charge.payment = payment

            chargeRepository.save(charge)
        }
    }
}
