package kr.co.vividnext.sodalive.creator.admin.calculate

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateContentDonationQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateLiveQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCumulativeSalesByContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateContentDonationQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateLiveQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCumulativeSalesByContentQueryData
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.QUseCanCalculate.useCanCalculate
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class CreatorAdminCalculateQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getCalculateLive(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): List<GetCalculateLiveQueryData> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                liveRoom.beginDateTime,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )

        return queryFactory
            .select(
                QGetCalculateLiveQueryData(
                    member.email,
                    member.nickname,
                    formattedDate,
                    liveRoom.title,
                    liveRoom.price,
                    useCan.canUsage,
                    useCanCalculate.id.count(),
                    useCanCalculate.can.sum()
                )
            )
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.room, liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                useCanCalculate.status.eq(UseCanCalculateStatus.RECEIVED)
                    .and(useCanCalculate.recipientCreatorId.eq(memberId))
                    .and(liveRoom.beginDateTime.goe(startDate))
                    .and(liveRoom.beginDateTime.loe(endDate))
            )
            .groupBy(liveRoom.id, useCan.canUsage)
            .orderBy(liveRoom.id.desc(), useCan.canUsage.desc(), formattedDate.desc())
            .fetch()
    }

    fun getCalculateContentListTotalCount(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): Int {
        val orderFormattedDate = getFormattedDate(order.createdAt)
        return queryFactory
            .select(audioContent.id)
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.order, order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCanCalculate.status.eq(UseCanCalculateStatus.RECEIVED)
                    .and(useCanCalculate.recipientCreatorId.eq(memberId))
                    .and(order.createdAt.goe(startDate))
                    .and(order.createdAt.loe(endDate))
            )
            .groupBy(audioContent.id, order.type, orderFormattedDate, order.can)
            .orderBy(member.id.desc(), orderFormattedDate.desc(), audioContent.id.asc())
            .fetch()
            .size
    }

    fun getCalculateContentList(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCalculateContentQueryData> {
        val orderFormattedDate = getFormattedDate(order.createdAt)
        return queryFactory
            .select(
                QGetCalculateContentQueryData(
                    member.nickname,
                    audioContent.title,
                    getFormattedDate(audioContent.createdAt),
                    orderFormattedDate,
                    order.type,
                    order.can,
                    order.id.count(),
                    order.can.sum()
                )
            )
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.order, order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCanCalculate.status.eq(UseCanCalculateStatus.RECEIVED)
                    .and(useCanCalculate.recipientCreatorId.eq(memberId))
                    .and(order.createdAt.goe(startDate))
                    .and(order.createdAt.loe(endDate))
            )
            .groupBy(audioContent.id, order.type, orderFormattedDate, order.can)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc(), orderFormattedDate.desc(), audioContent.id.asc())
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }

    fun getCumulativeSalesByContentTotalCount(memberId: Long): Int {
        return queryFactory
            .select(audioContent.id)
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(audioContent.member.id.eq(memberId))
            .groupBy(member.id, audioContent.id, order.can)
            .fetch()
            .size
    }

    fun getCumulativeSalesByContent(
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCumulativeSalesByContentQueryData> {
        return queryFactory
            .select(
                QGetCumulativeSalesByContentQueryData(
                    member.nickname,
                    audioContent.title,
                    getFormattedDate(audioContent.createdAt),
                    order.type,
                    order.can,
                    order.id.count(),
                    order.can.sum()
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(audioContent.member.id.eq(memberId))
            .groupBy(member.id, audioContent.id, order.type, order.can)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc(), audioContent.id.desc())
            .fetch()
    }

    fun getCalculateContentDonationListTotalCount(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): Int {
        val donationFormattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(audioContent.id)
            .from(useCan)
            .innerJoin(useCan.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.DONATION))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(audioContent.member.id.eq(memberId))
            )
            .groupBy(donationFormattedDate, audioContent.id)
            .fetch()
            .size
    }

    fun getCalculateContentDonationList(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCalculateContentDonationQueryData> {
        val donationFormattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(
                QGetCalculateContentDonationQueryData(
                    member.nickname,
                    audioContent.title,
                    audioContent.price,
                    getFormattedDate(audioContent.createdAt),
                    donationFormattedDate,
                    useCan.id.count(),
                    useCan.can.add(useCan.rewardCan).sum()
                )
            )
            .from(useCan)
            .innerJoin(useCan.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.DONATION))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(audioContent.member.id.eq(memberId))
            )
            .groupBy(donationFormattedDate, audioContent.id)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.asc(), donationFormattedDate.desc())
            .fetch()
    }
}
